@@ -1,4 +1,5 @@
 package Qt::slots;
+use Carp;
 #
 # Proposed usage:
 #
@@ -13,7 +14,7 @@ package Qt::slots;
 sub import {
     no strict 'refs';
     my $self = shift;
-    my $caller = (caller)[0];
+    my $caller = $self eq "Qt::slots" ? (caller)[0] : $self;
     my $parent = ${ $caller . '::ISA' }[0];
     my $parent_qt_invoke = $parent . '::qt_invoke';
 
@@ -32,13 +33,25 @@ sub import {
 #    } unless defined &{ $caller . '::qt_invoke' };
 
     my $meta = \%{ $caller . '::META' };
+    croak "Odd number of arguments in slot declaration" if @_%2;
     my(%slots) = @_;
     for my $slotname (keys %slots) {
 	my $slot = { name => $slotname };
 	my $args = $slots{$slotname};
 	$slot->{arguments} = [map { s/\s(?=[*&])//; { type => $_, name => "" } } @$args];
 	my $arglist = join ',', @$args;
+
 	$slot->{prototype} = $slotname . "($arglist)";
+        if ( exists $meta->{slot}{$slotname} ) {
+           (my $s1 = $slot->{prototype}) =~ s/\s+//g;
+           (my $s2 = $meta->{slot}{$slotname}{prototype})  =~ s/\s+//g; 
+           if( $s1 ne $s2 ) {
+               warn( "Slot declaration:\n\t$slot->{prototype}\nwill override ".
+                     "previous declaration:\n\t$meta->{slot}{$slotname}{prototype}");
+           } else {
+               next;
+           }      
+        }        
 	$slot->{returns} = 'void';
 	$slot->{method} = $slotname;
 	push @{$meta->{slots}}, $slot;
