@@ -20,6 +20,8 @@ package kalyptusCxxToSmoke;
 
 use File::Path;
 use File::Basename;
+use constant numSourceFiles => 20; # Total number of generated source files.
+                                   # All classes will be distributed across those.
 
 use Carp;
 use Ast;
@@ -180,7 +182,30 @@ sub writeDoc
 	print STDERR "Writing x_*.cpp...\n";
 
 	# Generate x_*cpp file for each class
-	Iter::LocalCompounds( $rootnode, sub { writeClassDoc( shift ); } );
+
+        my $numclasses;
+        Iter::LocalCompounds( $rootnode, sub { $numclasses++ } );
+        my $classperfile = int($numclasses/numSourceFiles);
+        print STDERR "Total number of classes: ". $numclasses ."\n" if $debug;
+        my $nodelist = [];
+        my $currentfile = 1;
+        my $currentclass = 1;
+        Iter::LocalCompounds( $rootnode, sub { 
+                   push @$nodelist, shift;
+                   if(@$nodelist == $classperfile and $currentfile != numSourceFiles)
+                   {
+                       print STDERR "Calling writeClassDoc for ". (scalar @$nodelist) . " classes\n" if $debug;
+                       writeClassDoc( $nodelist );
+                       $currentfile++;
+                       $nodelist = []     
+                   }
+                   if(@$nodelist and $currentclass == $numclasses)
+                   {    
+                       print STDERR "Calling writeClassDoc for remaining ". (scalar @$nodelist) . " classes\n" if $debug;
+                       writeClassDoc( $nodelist )
+                   }
+                   $currentclass++
+        });
 
 	print STDERR "Done.\n";
 }
@@ -203,6 +228,9 @@ sub preParseClass
 	    $className eq 'QCString' ||
 	    # Don't map classes which are really arrays
 	    $className eq 'QStringList' ||
+            $className eq 'QCanvasItemList' ||
+            $className eq 'QWidgetList' ||
+            $className eq 'QObjectList' ||
 	    $className eq 'QStrList' ||
 	    # Those are template related
             $className eq 'QTSManip' || # cause compiler errors with several gcc versions
@@ -307,6 +335,7 @@ sub preParseClass
 		     or $arg->{ArgType} eq 'EncoderFn' # QFile's callback
 		     or $arg->{ArgType} =~ /bool \(\*\)\(QObject/ # QMetaObject's ctor
 		     or $arg->{ArgType} eq 'QtStaticMetaObjectFunction' # QMetaObjectCleanUp's ctor with func pointer
+		     or $arg->{ArgType} eq 'const QTextItem&' # ref to a private class in 3.2.0b1
 		     or $arg->{ArgType} eq 'FILE*' # won't be able to handle that I think
 		) {
 		    $m->{NodeType} = 'deleted';
@@ -361,8 +390,10 @@ sub preParseClass
 
 	print STDERR "$className: ctor count: $constructorCount, hasPublicProtectedConstructor: $hasPublicProtectedConstructor, hasCopyConstructor: $hasCopyConstructor:, defaultConstructor: $defaultConstructor, hasPublicDestructor: $hasPublicDestructor, hasPrivatePureVirtual:$hasPrivatePureVirtual\n" if ($debug);
 
+	my $isGlobalSpace = ($className eq $main::globalSpaceClassName);
+
 	# Note that if the class has _no_ constructor, the default ctor applies. Let's even generate it.
-	if ( !$constructorCount && $defaultConstructor eq 'none' && !$hasPrivatePureVirtual ) {
+	if ( !$constructorCount && $defaultConstructor eq 'none' && !$hasPrivatePureVirtual && !$isGlobalSpace ) {
 	    # Create a method node for the constructor
 	    my $methodNode = Ast::New( $classNode->{astNodeName} );
 	    $methodNode->AddProp( "NodeType", "method" );
@@ -380,7 +411,7 @@ sub preParseClass
 	}
 
 	# Also, if the class has no explicit destructor, generate a default one.
-	if ( !$hasDestructor && !$hasPrivatePureVirtual ) {
+	if ( !$hasDestructor && !$hasPrivatePureVirtual && !$isGlobalSpace ) {
 	    my $methodNode = Ast::New( "$classNode->{astNodeName}" );
 	    $methodNode->AddProp( "NodeType", "method" );
 	    $methodNode->AddProp( "Flags", "" );
@@ -412,6 +443,7 @@ sub preParseClass
 	$classNode->AddProp( "HasCopyConstructor", $hasCopyConstructor );
 }
 
+
 sub propagateCanBeCopied($)
 {
 	my $classNode = shift;
@@ -453,56 +485,67 @@ sub propagateCanBeCopied($)
 
 =head2 writeClassDoc
 
-	Called by writeDoc for each class to be written out
+	Called by writeDoc for each series of classes to be written out
 
 =cut
 
-sub writeClassDoc
-{
-	my( $node ) = @_;
-	my $className = join( "::", kdocAstUtil::heritage($node) );
-	# Makefile doesn't like '::' in filenames, so use __
-	my $fileName  = join( "__", kdocAstUtil::heritage($node) );
-	print "Enter: $className\n" if $debug;
+BEGIN {
 
-	# Write out the x_*.cpp file
-	my $file = "$outputdir/x_$fileName.cpp";
-	open( XCPPFILE, ">$file" ) || die "Couldn't create $file\n";
-	print STDERR "Writing x_$fileName.cpp\n" if ($debug);
+my $fhn =1; # static
 
-	print XCPPFILE "//Auto-generated by $0. DO NOT EDIT.\n";
- 	print XCPPFILE "#include <smoke.h>\n";
- 	print XCPPFILE "#include <${libname}_smoke.h>\n";
+  sub writeClassDoc
+  {
+	my $nodelist = shift;
+        my $file = "$outputdir/x_${fhn}.cpp";
+        open( my $fh, ">$file" ) || die "Couldn't create $file\n";
 
-        my ($methodCode, $switchCode, $addInclude) = generateAllMethods( $node );
+	print $fh "//Auto-generated by $0. DO NOT EDIT.\n";
+ 	print $fh "#include <smoke.h>\n";
+ 	print $fh "#include <${libname}_smoke.h>\n";
 
-	foreach my $incl (keys %{$addInclude}) {
+        my @code;
+        for my $node ( @$nodelist )
+        {
+            push @code, [generateAllMethods( $node )]
+        }
+        my %includes;
+        map { for my $incl (keys %{$_->[2]}){ $includes{$incl}++ } } @code;
+
+	foreach my $incl (keys %includes) {
 	    die if $incl eq '';
-	    print XCPPFILE "#include <$incl>\n";
+	    print $fh "#include <$incl>\n";
 	}	
-	print XCPPFILE "\n";
-	print XCPPFILE "class x_$fileName "; # currently xClassName == x_$fileName
-	print XCPPFILE ": public $className " if $node->{BindingDerives};
-        print XCPPFILE "{\n";
-        print XCPPFILE $methodCode;
-        print XCPPFILE "};\n";
-	if(keys %{$node->{enumerations}}) {
-	    print XCPPFILE "void xenum_$fileName(Smoke::EnumOperation xop, Smoke::Index xtype, void *&xdata, long &xvalue) {\n";
-	    print XCPPFILE "    x_$fileName\::xenum_operation(xop, xtype, xdata, xvalue);\n";
-	    print XCPPFILE "}\n";
-	}
-	print XCPPFILE "void xcall_$fileName(Smoke::Index xi, void *obj, Smoke::Stack args) {\n";
-	print XCPPFILE $switchCode;
-	print XCPPFILE "}\n";
-
+	print $fh "\n";
+        for my $c( 0..$#code )
+        {
+           my ($methodCode, $switchCode, $incl) = @{ $code[$c] };
+           my $node = $$nodelist[$c];
+           my $className = join( "::", kdocAstUtil::heritage($node) );
+           my $legacyClassName = join( "__", kdocAstUtil::heritage($node) );
+           print $fh "class x_$legacyClassName ";
+           print $fh ": public $className " if $node->{BindingDerives};
+           print $fh "{\n";
+           print $fh $methodCode;
+           print $fh "};\n";
+	   if(keys %{$node->{enumerations}}) {
+	        print $fh "void xenum_${legacyClassName}(Smoke::EnumOperation xop, Smoke::Index xtype, void *&xdata, long &xvalue) {\n";
+	        print $fh "    x_${legacyClassName}\::xenum_operation(xop, xtype, xdata, xvalue);\n";
+	        print $fh "}\n";
+	   }
+	   print $fh "void xcall_${legacyClassName}(Smoke::Index xi, void *obj, Smoke::Stack args) {\n";
+	   print $fh $switchCode;
+	   print $fh "}\n\n";
+        }
 	#if ( $className =~ /^(QBrush|QColor|QCursor|QFont|QImage|QPalette|QPixmap|QPoint|QPointArray|QRect|QRegion|QSize|QWMatrix)$/ ) {
 	#	print XCPPFILE "    const char *{serial} operator << () const : pig_serialize(\$this);\n";
 	#	print XCPPFILE "    void operator >> (const char *{serial}) : pig_deserialize(\$this, \$1);\n";
 	#}
 
-	close XCPPFILE;
-}
+	close $fh;
+        $fhn++
+  }
 
+}
 
 # Generate the prototypes for a method (one per arg with a default value)
 # Helper for makeprotos
@@ -973,7 +1016,20 @@ sub generateMethod($$$)
 	    } else {
 	        $methodCode .= $returnType . " xret = " if $returnType;
 	        $methodCode .= "$this\->" unless $isStatic;
-	        $methodCode .= "$className\::$name(@castedArgList[0..$#argTypeList]);\n";
+		if ($className ne $main::globalSpaceClassName) {
+		    $methodCode .= "$className\::$name(@castedArgList[0..$#argTypeList]);\n";
+		} elsif ($name =~ /^operator\s?\W+/) {
+		    ( my $op = $name ) =~ s/^operator(.*)$/$1/;
+		    if (scalar(@argTypeList) == 2) {
+			$methodCode .= "(@castedArgList[0] $op @castedArgList[1]);\n"; # a + b
+		    } elsif (scalar(@argTypeList) == 1) {
+			$methodCode .= "($op@castedArgList[0]);\n"; # -a
+		    } else {
+			die "shouldn't reach here!";
+		    }
+		} else {
+		    $methodCode .= "$name(@castedArgList[0..$#argTypeList]);\n";
+		}
 	        $methodCode .= "\t" . $xretCode if $returnType;
 		# To avoid unused parameter warning, add this to void methods:
 		$methodCode .= "\t(void)x; // noop (for compiler warning)\n" unless $returnType;
@@ -1103,21 +1159,33 @@ sub generateAllMethods
     #my $className = $classNode->{astNodeName};
     my $className = join( "::", kdocAstUtil::heritage($classNode) );
     my $xClassName = "x_" . join( "__", kdocAstUtil::heritage($classNode) );
+    my $isGlobalSpace = ($xClassName eq ("x_".$main::globalSpaceClassName));
     my $sourcename = $classNode->{Source}->{astNodeName};
+   
     $sourcename =~ s!.*/(.*)!$1!m;
     die "Empty source name for $classNode->{astNodeName}" if ( $sourcename eq '' );
 
     my %addInclude = ( $sourcename => 1 );
 
-    if(! $classNode->{BindingDerives}) {
-	$methodCode .= "private:\n";
-	$methodCode .= "    $className *xthis;\n";
-	$methodCode .= "public:\n";
-	$methodCode .= "    $xClassName\(void *x) : xthis(($className*)x) {}\n";
-	$switchCode .= "    $xClassName xtmp(obj), *xself = &xtmp;\n";
+    if (!$isGlobalSpace) {
+        if(! $classNode->{BindingDerives}) {
+            $methodCode .= "private:\n";
+            $methodCode .= "    $className *xthis;\n";
+            $methodCode .= "public:\n";
+            $methodCode .= "    $xClassName\(void *x) : xthis(($className*)x) {}\n";
+            $switchCode .= "    $xClassName xtmp(obj), *xself = &xtmp;\n";
+        } else {
+            $switchCode .= "    $xClassName *xself = ($xClassName*)obj;\n";
+            $methodCode .= "public:\n";
+        }
     } else {
-	$switchCode .= "    $xClassName *xself = ($xClassName*)obj;\n";
+        my $s;
+        for my $sn( @{$classNode->{Sources}} ) {
+		($s = $sn->{astNodeName}) =~ s!.*/(.*)!$1!m;
+		$addInclude{ $s } = 1;
+        }
 	$methodCode .= "public:\n";
+	$switchCode .= "    (void) obj;\n";
     }
     $switchCode .= "    switch(xi) {\n";
 
@@ -1173,7 +1241,9 @@ sub generateAllMethods
     #	$methodCode .= "    virtual ~$xClassName() {}\n";
     #}
     # We generate a dtor though, because we might want to add stuff into it
-    $methodCode .= "    ~$xClassName() { ${libname}_Smoke->binding->deleted($classNode->{ClassIndex}, (void*)this); }\n";
+    if ( !$isGlobalSpace ) {
+        $methodCode .= "    ~$xClassName() { ${libname}_Smoke->binding->deleted($classNode->{ClassIndex}, (void*)this); }\n";
+    }
 
     if ($classNode->{CanBeInstanciated} and $classNode->{HasPublicDestructor}) {
 	die "$className destructor: methodNumber=$methodNumber != case entry=".$classNode->{case}{"~$className()"}."\n"
@@ -1507,6 +1577,7 @@ sub writeSmokeDataFile($) {
 
     print OUT "\n";
     print OUT "#include <smoke.h>\n\n";
+    print OUT "#include <qt_smoke.h>\n\n";
 
     # gcc optimizes this method like crazy. switch() is godly
     print OUT "static void *${libname}_cast(void *xptr, Smoke::Index from, Smoke::Index to) {\n";
@@ -1822,8 +1893,7 @@ sub writeSmokeDataFile($) {
     print OUT "static Smoke::Index ${libname}_argumentList[] = {\n";
     my $argListCount = 0;
     for my $args (sort keys %arglist) {
-	my @dunnohowtoavoidthat = split(',',$args);
-	my $numTypes = $#dunnohowtoavoidthat;
+	my $numTypes = scalar(split ',', $args);
 	if ($args eq '') {
 	    print OUT "\t0,\t//0  (void)\n";
 	} else {
@@ -1831,7 +1901,7 @@ sub writeSmokeDataFile($) {
 	    print OUT "\t$args, 0,\t//$argListCount  $arglist{$args}  \n";
 	}
 	$arglist{$args} = $argListCount;      # Register proper index in argList
-	$argListCount += $numTypes + 2;       # Move forward by as much as we wrote out
+	$argListCount += $numTypes + 1;       # Move forward by as much as we wrote out
     }
     print OUT "};\n\n";
 
